#include "TPhDetFamily.h"
#include <fstream.h>
#include <stdlib.h>
#include <string.h>

//#define DEBUG

ClassImp(TPhDetFamily)


////////////////////////////////////////////////////////////
//
// TPhDetFamily is a PHOBOS base data class providing storage 
// for one DetFamily All detector families inherit from this 
// class
//
////////////////////////////////////////////////////////////

TPhDetFamily::TPhDetFamily()
{
// Constructor. Set lists to NULL; Sets Iterators to NULL;

    fDetectorList = NULL;
	fDetectorIterator = NULL;
}

TPhDetFamily::~TPhDetFamily()
{
// Destructor. Free all memory, destroying all the family members

	if(fDetectorIterator) delete fDetectorIterator;
	fDetectorIterator = NULL;
    if(fDetectorList)
    {
        fDetectorList->Delete();
        delete fDetectorList;
		fDetectorList = NULL;
    }
}

void TPhDetFamily::AddDetector(TPhDetector * detector)
{
// Add a pointer to a TPhDetector object to the fDetectorList
// array. For the first Detector, fDetectorList is allocated 
// here.

   if(!fDetectorList)
       fDetectorList = new TObjArray();
#ifdef DEBUG
   cout << "Adding Detector:" << detector->GetDetID() << endl;
#endif
   fDetectorList->Add(detector);
}


Int_t TPhDetFamily::DropDetector(TPhDetector* detector)
{
   // Remove Detector from array.
   // if deleting from wrong array return -1;

   if (!detector) return 0; // no hit nothing to delete no error

   if (!fDetectorList)
   {
	   return -1; // no list to delete from;
   }
   else
   {
	if ((fDetectorList->IndexOf(detector) - 
		 fDetectorList->LowerBound()) == -1)
	{
		return -1; // object not in list
	}
	else
	{
	   fDetectorList->Remove(detector);
       delete detector;
	   detector = NULL;
	   return 0;
	}

   }

}

Int_t TPhDetFamily::DropDetector(Int_t id)
{
   // Remove detector from array.
   // if detector doesn't exist give error
  TPhDetector * detector = GetDetector(id);
  if (detector)
  {
	  return DropDetector(detector);
  }
  else
  { 
	  return -1;
  }
}


Int_t TPhDetFamily::GetNDetectors()
{
// Return number of detectors in the DetFamily, if any.

    if(fDetectorList)
        return fDetectorList->GetEntries();
    else
        return 0;
}


TPhDetector* TPhDetFamily::GetNextDetector()
{
	// Iterator over all Detectors in the array.  
	// Uses fDetectorIterator
	// Cannot be nested
	// Use GetDetectorList and use TObjArrayIterator class. for nesting
	// Returns zero if no more hits in array

    if(fDetectorList)
    {
      if(!fDetectorIterator)
	  {
		  SetupDetectorIterator();
		  ResetDetectorIterator();
	  }
      return (TPhDetector *) fDetectorIterator->Next();
    }
    else
    {
        return 0;
    }
}


void TPhDetFamily::ResetDetectorIterator()
{
	// Resets the Detector list Iterator.
	// If not present it creates a new Iterator


	if(fDetectorList)
	{
      if(!fDetectorIterator)
	  {
        fDetectorIterator = fDetectorList->MakeIterator();
	  }
	  else
	  {
        fDetectorIterator->Reset();
	  }
	}
}

void TPhDetFamily::SetupDetectorIterator(Bool_t dir)
{
	// Creates a new Detector list Iterator.
	// If one already present it deletes it and makes a new one	
	// By default the Iteratoration direction is kIteratorForward. 
    // To go backward use kIteratorBackward.


	if(fDetectorIterator)
	{
		delete fDetectorIterator;
		fDetectorIterator = NULL;
	}
	if(fDetectorList)
	{
        fDetectorIterator = fDetectorList->MakeIterator(dir);
	}
}

TPhDetector * TPhDetFamily::GetDetector(Int_t ID)
{
// Get the Family member with given ID number
// Resets the DetectorIterator
	
    if(fDetectorList)
    {
       ResetDetectorIterator();
	   TPhDetector* detector= GetNextDetector();
	   while (detector && detector->GetDetID() != ID)
	   {
		   detector = GetNextDetector();
	   }
#ifdef DEBUG2
	   cout << "Looking for detector ID " << ID;
	   if (detector) 
	   {
        cout   << " found " << detector->GetDetID() << endl;
	   }
	   else
	   {
		  cout << " found none." << endl;
	   }

#endif
	   return detector;
	   
    }
    else
    {
        return 0;
    }
}

Bool_t TPhDetFamily::Draw(Int_t color)
{
    TPhDetector * det;  

	ResetDetectorIterator();
    while(det = GetNextDetector())
    {
		cout << "Drawing Detector #" << det->GetDetID() << endl;
        det->Draw(color);
    }
    return kTRUE;
}

Bool_t TPhDetFamily::DrawPoint(Int_t color)
{
    TPhDetector * det;  

	ResetDetectorIterator();
    while(det = GetNextDetector())
    {
		cout << "Drawing Detector #" << det->GetDetID() << endl;
        det->DrawPoint(color);
    }
    return kTRUE;
}

void TPhDetFamily::DropAllHits()
{
	// Drops all hits in the family.
	// Resets all iterators.
	ResetDetectorIterator();
	TPhDetector * det = GetNextDetector();
    while (det)
	{
	 det->DropAllHits();
	 det = GetNextDetector();
	}
}

void TPhDetFamily::FillData()
{
	// Call FillData on each detector
	ResetDetectorIterator();
	TPhDetector * det = GetNextDetector();
    while (det)
	{
	 det->FillData();
	 det = GetNextDetector();
	}
}
void TPhDetFamily::CalcPed()
{
	// Call CalcPed on each Detector
	ResetDetectorIterator();
	TPhDetector * det = GetNextDetector();
    while (det)
	{
	 det->CalcPed();
	 det = GetNextDetector();
	}
}
void TPhDetFamily::AddPed()
{
	// Call AddPed on each Detector
	ResetDetectorIterator();
	TPhDetector * det = GetNextDetector();
    while (det)
	{
	 det->AddPed();
	 det = GetNextDetector();
	}
}


Bool_t   TPhDetFamily::ReadPedFile(char *inputname1)
{  // read pedestals if no file give make blank


	Bool_t Zero = kFALSE;

	ifstream infile1;

    if(!inputname1)
    {
		cout << "No pedistats setting to Zero" << endl;
		Zero = kTRUE;
	}
	else
	{
		infile1.open(inputname1, ios::in | ios::nocreate);
	}

	if (!infile1)
    {
	    cerr << "Can not open '" << inputname1 << "' for input." << endl;
	    cerr << "Setting pedestals to zero." << endl;
	    Zero = kTRUE;
    }
 
	Int_t nchan;
	Int_t chan;
	TPhDetector * det;

#ifdef DEBUG
		cout << " Reading Ped Descritption file" << endl;
#endif
	
	ResetDetectorIterator();
	det =  (GetNextDetector());
	while (det)
	{
		Int_t NPedEvents;
		Float_t ped;
		nchan = det->GetNPads();
#ifdef DEBUG
		cout << "About to input values" << endl;
#endif
		if (!(infile1 >> NPedEvents))
		{
			Zero = kTRUE;
			NPedEvents = 0;
		}

		det->SetNPedEvent(NPedEvents);

		for (Int_t i = 0; i < nchan; i++)
		{
			if (Zero)
			{ 
				det->SetPed(i,0);
			}
			else if (infile1 >> chan >> ped)
			{
				if (chan == i)
				{
					det->SetPed(i,ped);
				}
				else
				{
					cerr << "pedestal file is corruput" << endl;
				}
			}
			else
			{
				det->SetPed(i,0);
			}
		}

#ifdef DEBUG
		cout << "Values Stored" << endl;
		Float_t * val;
		val = det->GetPedData();
		for (i = 0; i < nchan; i++)
		{
			cout << val[i] << ":";
		}
		cout << endl;
#endif
		det = GetNextDetector();

	}
#ifdef DEBUG
		cout << "Done With Reading" << endl;
	
#endif

    infile1.close();
	return kTRUE;
}

Bool_t   TPhDetFamily::WritePedFile(char *inputname1)
{  
	char filename[80];
	char runnumber[10];
	char name[10];

	ofstream outfile1;

    if(!inputname1)
    {
		itoa(fEvent->GetRun(),runnumber,10);
		itoa(fID,name,10);
		strcat(filename, "Ped_Det");
		strcat(filename, name);
		strcat(filename, "_Run");
		strcat(filename, runnumber);
		cout << "Filename is: " << filename;
		outfile1.open(filename, ios::out);
	}
	else
	{
		outfile1.open(inputname1, ios::out);
	}
    
		
   if (!outfile1)
   {
	   cerr << "Can not open '" << inputname1 << "' for output." << endl;
	   return kFALSE;
   }
   
   Int_t nchan;
   TPhDetector * det;

#ifdef DEBUG
	cout << " Writing Ped Descritption file" << endl;
#endif
	
	ResetDetectorIterator();
	det =  (GetNextDetector());
	while (det)
	{
		Int_t npedevent;
		npedevent = det->GetNPedEvent();
		outfile1 << npedevent << endl;
		nchan = det->GetNPads();
		for (Int_t i = 0; i < nchan; i++)
		{
			outfile1 << i << ' ' << det->GetPed(i) << endl;
		}

		det = GetNextDetector();
	}
	return kTRUE;
}
