
#include <TView.h>
#include <TCanvas.h>
#include <iostream.h>

#include "TPhEvent.h"

//#define DEBUG

ClassImp(TPhEvent)


////////////////////////////////////////////////////////////
//
// TPhEvent is a PHOBOS base data class providing storage 
// for one Event All hits/Familys/Family familys are
// contained.
// 
// An Event has a list of Family Families.
// One Family per Family Type.
//
// Family Families contian of list of all detectors in
// their family
//  
// Each detector contains all the raw and calibrated hits 
// associated with it
//
////////////////////////////////////////////////////////////

TPhEvent::TPhEvent()
{
// Constructor. Set lists to NULL; Sets Iterators to NULL;

    fFamilyList = NULL;
	fFamilyIterator = NULL;
}

TPhEvent::~TPhEvent()
{
// Destructor. Free all memory, destroying all the family members

    if(fFamilyList)
    {
		if(fFamilyIterator) delete fFamilyIterator;
		fFamilyIterator = NULL;
        fFamilyList->Delete();
        delete fFamilyList;
		fFamilyList = NULL;
    }
}

void TPhEvent::AddFamily(TPhDetFamily * Family)
{
// Add a pointer to a TPhDetFamily object to the fFamilyList
// array. For the first Family, fFamilyList is allocated 
// here.

   if(!fFamilyList)
       fFamilyList = new TObjArray();
   if(GetFamily(Family->GetDetTypeID()))
   {
// Error adding falmily that already exists!
	   cerr << "Adding Family that already exists" << endl;
   }
   else
   {
   fFamilyList->Add(Family);
   Family->SetEvent(this);
   }
}


Int_t TPhEvent::DropFamily(TPhDetFamily* Family)
{
   // Remove Family from array.
   // if deleting from wrong array return -1;

   if (!Family) return 0; // no hit nothing to delete no error

   if (!fFamilyList)
   {
	   return -1; // no list to delete from;
   }
   else
   {
	if ((fFamilyList->IndexOf(Family) - 
		 fFamilyList->LowerBound()) == -1)
	{
		return -1; // object not in list
	}
	else
	{
	   fFamilyList->Remove(Family);
       delete Family;
	   Family = NULL;
	   return 0;
	}

   }

}

Int_t TPhEvent::DropFamily(Int_t id)
{
   // Remove Family from array.
   // if Family doesn't exist give error
  TPhDetFamily * Family = GetFamily(id);
  if (Family)
  {
	  return DropFamily(Family);
  }
  else
  { 
	  return -1;
  }
}


Int_t TPhEvent::GetNFamily()
{
// Return number of Familys in the Event, if any.

    if(fFamilyList)
        return fFamilyList->GetEntries();
    else
        return 0;
}


TPhDetFamily* TPhEvent::GetNextFamily()
{
	// Iterator over all Familys in the array.  
	// Uses fFamilyIterator
	// Cannot be nested
	// Use GetFamilyList and use TObjArrayIterator class. for nesting
	// Returns zero if no more hits in array

    if(fFamilyList)
    {
      if(!fFamilyIterator)
	  {
		  SetupFamilyIterator();
	  }
      return (TPhDetFamily *) fFamilyIterator->Next();
    }
    else
    {
        return 0;
    }
}


void TPhEvent::ResetFamilyIterator()
{
	// Resets the Family list Iterator.
	// If not present it creates a new Iterator


	if(fFamilyList)
	{
      if(!fFamilyIterator)
	  {
        fFamilyIterator = fFamilyList->MakeIterator();
	  }
	  else
	  {
        fFamilyIterator->Reset();
	  }
	}
}

void TPhEvent::SetupFamilyIterator(Bool_t dir)
{
	// Creates a new Family list Iterator.
	// If one already present it deletes it and makes a new one	
	// By default the Iteratoration direction is kIteratorForward. 
    // To go backward use kIteratorBackward.


	if(fFamilyIterator)
	{
		delete fFamilyIterator;
		fFamilyIterator = NULL;
	}
	if(fFamilyList)
	{
        fFamilyIterator = fFamilyList->MakeIterator(dir);
	}
}

TPhDetFamily *TPhEvent::GetFamily(Int_t ID)
{
// Get the Family member with given TypeID number
// Resets the FamilyIterator
	
    if(fFamilyList)
    {
       ResetFamilyIterator();
	   TPhDetFamily* Family= GetNextFamily();
	   while (Family && Family->GetDetTypeID() != ID)
	   {
#ifdef DEBUG
		   cout << "Looking for " << ID << " At " << Family->GetDetTypeID() << endl;
#endif
		   Family = GetNextFamily();
	   }
#ifdef DEBUG
		   cout << "Looking for " << ID;
		   if (Family) cout << " Found " << Family->GetDetTypeID() << endl;
		   else cout << " Found nothing" << endl;
#endif
	   return Family;
	   
    }
    else
    {
        return 0;
    }
}

Int_t TPhEvent::GetNFamilyMembers(Int_t TypeID)
{
    TPhDetFamily * family = GetFamily(TypeID);
	if (family)
		return family->GetNDetectors();
    else
        return 0;
}


Bool_t TPhEvent::Draw(Int_t color)
{
    TPhDetFamily *det;  

    TCanvas *canvas  = new TCanvas("c1","Everything",700,700);
    canvas->Draw();

    TView *view = new TView(1);
    view->SetRange(-50.,
		-50.,
		-10.,
		50.,
		50.,
		10.);

	ResetFamilyIterator();
    while(det = GetNextFamily())
    {
		cout << "Drawing Family # " << det->GetID() << endl;
        det->Draw(color);
    }

    return kTRUE;
}

Bool_t TPhEvent::DrawPoint(Int_t color)
{
    TPhDetFamily *det;  

    TCanvas *canvas  = new TCanvas("c2","Everything Point",700,700);
    canvas->Draw();

    TView *view = new TView(1);
    view->SetRange(
		-50.,
		-50.,
		-10.,
		 50.,
		 50.,
		 10.);

	ResetFamilyIterator();
    while(det = GetNextFamily())
    {
		cout << "Drawing Family # " << det->GetID() << endl;
        det->DrawPoint(color);
    }

    return kTRUE;
}

void TPhEvent::DropAllHits()
{
	// Drops all hits in the event.
	// Resets all iterators.
	ResetFamilyIterator();
	TPhDetFamily * fam = GetNextFamily();
    while (fam)
	{
	 fam->DropAllHits();
	 fam = GetNextFamily();
	}
}

void TPhEvent::FillData()
{
	// Call Fill Data for every family
	ResetFamilyIterator();
	TPhDetFamily * fam = GetNextFamily();
    while (fam)
	{
	 fam->FillData();
	 fam = GetNextFamily();
	}
}

void TPhEvent::CalcPed()
{
	// Call CalcPed for every family
	ResetFamilyIterator();
	TPhDetFamily * fam = GetNextFamily();
    while (fam)
	{
	 fam->CalcPed();
	 fam = GetNextFamily();
	}
}

void TPhEvent::AddPed()
{
	// Call AddPed for every family
	ResetFamilyIterator();
	TPhDetFamily * fam = GetNextFamily();
    while (fam)
	{
	 fam->AddPed();
	 fam = GetNextFamily();
	}
}



