
///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhWA98Family                                                  //
//                                                                   //
//    WA98 DetFamily base class                                      //
///////////////////////////////////////////////////////////////////////

// Phob Classes
#include "TPhWA98Fam.h"
#include "TPhWA98Pad.h"

// Root Classes
#include <TObject.h>

// General Classes
#include <math.h>
#include <iostream.h>
#include <fstream.h>

//#define DEBUG

ClassImp(TPhWA98Family)

// Description of the WA98 Detector family
// Contains most of the geometry specific stuff

// Initialize TPhWA98Family constants
const Int_t TPhWA98Family::kFamID(12);
const Int_t TPhWA98Family::kDataSize(4096);
const Int_t TPhWA98Family::kNumRadius(22);

TPhWA98Family::TPhWA98Family()
{
  PadRadiiLow  = new Double_t[kNumRadius];
  PadRadiiHigh = new Double_t[kNumRadius];
	fDetTypeID = kFamID;
	fID        = kFamID;
	{
		for (Int_t i=0; i< kNumRadius; i++)
		{
			PadRadiiLow[i]  = 5. * (i-1);
		    PadRadiiHigh[i] = 5. * (i);
		}
	}
}

Bool_t   TPhWA98Family::ReadGeometry(Char_t *inputname1, Char_t *inputname2)
{
    Char_t*   defaultfile1 = "WA98Translat.dat";
    Char_t*   defaultfile2 = "WA98Radii.dat" ;
    Char_t* filename1;
    Char_t* filename2;


    if(!inputname1)
    {
		cout << "No WA98 Pad Description File! Trying:" << defaultfile1 << endl;
        filename1 = defaultfile1;
	}
	else
	{
		filename1 = inputname1;
    }

    
    ifstream infile1(filename1, ios::in | ios::nocreate);
	if (!infile1)
	{
		cerr << "Can not open '" << filename1 << "' for input." << endl;
		return kFALSE;
	}
 
    if(!inputname2)
    {
		cout << "No WA98 Radius description File! Trying:" << defaultfile2 << endl;
        filename2 = defaultfile2;
	}
	else
	{
		filename2 = inputname2;
    }

    
    ifstream infile2(filename2, ios::in | ios::nocreate);
	if (!infile2)
	{
		cerr << "Can not open '" << filename2 << "' for input." << endl;
		return kFALSE;
	}

    Int_t padRadius;
	Double_t low;
	Double_t high;


	for (Int_t counter = 1; counter<=kNumRadius; counter++)
	{
        if ((infile2 >> padRadius
		             >> low
		     		 >> high))
		{
	        if(counter != padRadius)
			{
			    cerr << "Error in WA98 radius description file." << endl;
		        return kFALSE;
			}
		    PadRadiiLow[counter - 1] = low;
		    PadRadiiHigh[counter - 1] = high;

		}
	    else
		{
			cerr << "Not enough lines in WA98 radius description file." << endl;
		}
	}

	if(fDetectorList)
    {
		fDetectorList->Delete();
		delete fDetectorList;
        fDetectorList = NULL;
    }
	fDetectorList = new TObjArray();

	Int_t channel;
	Int_t preamp;
	Int_t chipChannel;
	
	Int_t padPhi;
	
	Double_t centerRadius;
	Double_t centerPhi;
    Double_t pseudorapidity;
	Int_t quadrant;
	TPhDetector * det = NULL;
	Int_t oldPlane = 0;

	counter = 0;
	while ((infile1 >> channel
		           >> preamp
				   >> chipChannel
				   >> padRadius
				   >> padPhi
				   >> centerRadius
				   >> centerPhi
				   >> pseudorapidity
				   >> quadrant))
	{
		counter++;
		TPhWA98Pad* pad = new TPhWA98Pad();

		pad->SetPreampChip(preamp);
		pad->SetChipChannel(chipChannel);
		pad->SetPadRadius(padRadius);
		pad->SetPadPhi(padPhi);
		pad->SetCenterRadius(centerRadius);
		pad->SetCenterPhi(centerPhi);
		pad->SetPseudorapidity(pseudorapidity);
		pad->SetPlane(quadrant);

		if (oldPlane != quadrant)
		{

			det = GetDetector(quadrant);
			if (!det)
			{
#ifdef DEBUG
			cout << "Adding new detector #" << quadrant << endl;
#endif

			// Assumes channel numbers are in order
				det = new TPhDetector();
				det->SetDetID(quadrant);
				det->SetDetTypeID(kFamID);
				AddDetector(det);
				det->SetupAllData(kDataSize/4,channel);
			}
			oldPlane = quadrant;
		}
#ifdef DEBUG2
			cout << "Seting up Data #" << channel << endl;
#endif
        if (padRadius >= 0)
		{
    		pad->SetXcentP(centerRadius * cos(centerPhi * atan(1.) /45.));
	    	pad->SetYcentP(centerRadius * sin(centerPhi * atan(1.) /45.));
    		pad->SetXcent(pad->GetXcentP());
	    	pad->SetYcent(pad->GetYcentP());
	    	pad->SetZcent(0.);
	    	pad->SetWidth(centerRadius * atan(1.) /45. * 2.); // 2 degrees * radius
	    	pad->SetHeight(PadRadiiHigh[padRadius]-PadRadiiLow[padRadius]);
		
			pad->SetCornersX0(PadRadiiHigh[padRadius] * cos((centerPhi-1) * atan(1.) /45.));
	    	pad->SetCornersX1(PadRadiiHigh[padRadius] * cos((centerPhi+1) * atan(1.) /45.));
	    	pad->SetCornersX2(PadRadiiLow[padRadius]  * cos((centerPhi+1) * atan(1.) /45.));
	    	pad->SetCornersX3(PadRadiiLow[padRadius]  * cos((centerPhi-1) * atan(1.) /45.));
			pad->SetCornersY0(PadRadiiHigh[padRadius] * sin((centerPhi-1) * atan(1.) /45.));
	    	pad->SetCornersY1(PadRadiiHigh[padRadius] * sin((centerPhi+1) * atan(1.) /45.));
	    	pad->SetCornersY2(PadRadiiLow[padRadius]  * sin((centerPhi+1) * atan(1.) /45.));
	    	pad->SetCornersY3(PadRadiiLow[padRadius]  * sin((centerPhi-1) * atan(1.) /45.));
			pad->SetCornersZ0(Double_t(0.));
			pad->SetCornersZ1(Double_t(0.));
			pad->SetCornersZ2(Double_t(0.));
			pad->SetCornersZ3(Double_t(0.));
#ifdef DEBUG2
			cout << "Done Seting up Data #" << channel << endl;
#endif
 
	    }
		else
		{
#ifdef DEBUG2
			cout << "Skipping Pad #" << quadrant << endl;
#endif
 
			pad->SetActive(kFALSE);
		}

#ifdef DEBUG2
			cout << "Adding Pad #" << channel << endl;
#endif
		det->AddPad(pad,channel);
		
    }
	
	
		ResetDetectorIterator();
		det = GetNextDetector();
		while (det)
		{
#ifdef DEBUG
			cout << "Setting for Det #" << det->GetDetID() << endl;
#endif
			Int_t nchan = det->GetNPads();
			det = GetNextDetector();
		}
    infile1.close();
    infile2.close();

#ifdef DEBUG
			cout << "Done with WA98 Geometry" << endl;
#endif
    return kTRUE;
}